from collections import defaultdict

from discord import SyncWebhook
from solo.models import SingletonModel

from django.contrib.auth.models import User
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access Secret Santa"),
            ("manager", "Can Manage Secret Santa"),
        )


class Webhook(models.Model):
    name = models.CharField(
        _("Name"),
        max_length=100)
    url = models.URLField(
        _("URL"),
        max_length=200)
    enabled = models.BooleanField(default=True)

    def __str__(self) -> str:
        return self.name

    def send_embed(self, embed):
        webhook = SyncWebhook.from_url(self.url)
        webhook.send(embed=embed, username="Secret Santa")


class SecretSantaConfiguration(SingletonModel):
    delivery_updates = models.ManyToManyField(
        Webhook,
        verbose_name=_("Delivery Updates"),
        help_text=_("A Santee has marked they have received their gift")
    )

    def __str__(self) -> str:
        return "Secret Santa Configuration"

    class Meta:
        """
        Meta definitions
        """
        verbose_name = "Secret Santa Configuration"
        default_permissions = ()


class Year(models.Model):
    year = models.IntegerField(
        validators=[MinValueValidator(2000), MaxValueValidator(2050)],
    )
    open = models.BooleanField(
        _("Open to Applications"),
        default=True,
    )

    @property
    def members(self) -> int:
        try:
            return Application.objects.filter(year=self).count()
        except Exception:
            return 0

    def get_users_santee(self, user: User) -> User:
        return SantaPair.objects.get(year=self, santa=user).santee

    def __str__(self) -> str:
        return str(self.year)


class Application(models.Model):
    """An Application to join Secret Santa"""
    year = models.ForeignKey(
        Year,
        verbose_name=_("Year"),
        on_delete=models.CASCADE)
    user = models.ForeignKey(
        User,
        verbose_name=_("User"),
        on_delete=models.CASCADE,
        related_name="+")

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()
        unique_together = ['year', 'user']

    def __str__(self) -> str:
        return f"{self.year} - {self.user}"


class SantaPair(models.Model):
    """A pairing of Santa's and Santee's"""
    year = models.ForeignKey(
        Year,
        verbose_name=_("Year"),
        on_delete=models.CASCADE)
    santa = models.ForeignKey(
        User,
        verbose_name=_("Santa"),
        on_delete=models.CASCADE,
        related_name="+")
    santee = models.ForeignKey(
        User,
        verbose_name=_("Santee"),
        on_delete=models.CASCADE,
        related_name="+")
    delivered = models.BooleanField(
        _("Delivered"),
        default=False)

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()
        unique_together = [['year', 'santa'], ['year', 'santee']]

    def __str__(self) -> str:
        return f"{self.year} - {self.santa} -> {self.santee}"


class ActiveSecretSantaFilter(models.Model):
    """A filter to pass or fail if people have an active santee/santa relationship"""

    name = models.CharField(max_length=500)
    description = models.CharField(max_length=500)

    year = models.ForeignKey(Year, on_delete=models.CASCADE, help_text="What year to filter on.")

    reversed_logic = models.BooleanField(
        default=False, help_text="If set all members WITHOUT a secret santa application will pass the test.")

    def __str__(self) -> str:
        return f"{self.name}: {self.description}"

    def process_filter(self, user: User):
        return self.audit_filter([user])[user.id]["check"]

    def audit_filter(self, users):
        logic = self.reversed_logic
        users_in_year = set(
            list(
                Application.objects.filter(
                    year=self.year,
                    user__in=users
                ).values_list('user_id', flat=True)
            )
        )
        output = defaultdict(lambda: {"message": "", "check": logic})
        for u in users:
            c = u.id in users_in_year
            if c is not False:
                output[u.id] = {
                    "message": f"Signed up to Gift in {self.year}", "check": not logic}
                continue
            output[u.id] = {"message": f"Not Gifting in {self.year}", "check": logic}
        return output

    class Meta:
        """
        Meta definitions
        """
        default_permissions = ()
        verbose_name = "Smart Filter: Active Secret Santa"
        verbose_name_plural = verbose_name



from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.exceptions import ValidationError
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from allianceauth.services.hooks import get_extension_logger

from secretsanta.app_settings import (
    SECRETSANTA_GENERATE_PAIRS_PRIORITY, SECRETSANTA_NOTIFY_PRIORITY,
)
from secretsanta.models import Application, SantaPair, User, Year

from .tasks import generate_pairs, notify_outstanding_santees, notify_santas

logger = get_extension_logger(__name__)


def get_users_santee(year: Year, user: User) -> User | str:
    try:
        return SantaPair.objects.get(year=year, santa=user).santee
    except SantaPair.DoesNotExist:
        try:
            if Application.objects.filter(year=year, user=user).exists():
                return "Not Yet Assigned"
            else:
                ""
        except Exception:
            return ""
        return ""


def render_year(request, year) -> str:
    year_obj = Year.objects.get(year=year)

    context = {
        "year": year_obj,
        "application_exists": Application.objects.filter(year=year_obj, user=request.user).exists(),
        "santapair_exists": SantaPair.objects.filter(year=year_obj, santee=request.user).exists(),
        "gift_delivered": SantaPair.objects.filter(year=year_obj, santee=request.user, delivered=True).exists(),
        "users_santee": get_users_santee(year=year_obj, user=request.user),
    }
    return render_to_string("secretsanta/year.html", context, request)


@login_required
@permission_required("secretsanta.basic_access")
def index(request) -> HttpResponse:
    context = {'year_renders': []}
    for year in Year.objects.all().order_by("-year"):
        context['year_renders'].append(render_year(request, year=year.year))
    return render(request, "secretsanta/index.html", context)


@login_required
@permission_required("secretsanta.manager")
def pairs(request, year) -> HttpResponse:
    year_obj = Year.objects.get(year=year)
    context = {
        "year": year_obj,
        "pairs": SantaPair.objects.filter(year=year_obj)
    }
    return render(request, "secretsanta/pairs.html", context)


@login_required
@permission_required("secretsanta.manager")
def applications(request, year) -> HttpResponse:
    year_obj = Year.objects.get(year=year)
    context = {
        "year": year_obj,
        "applications": Application.objects.filter(year=year_obj)
    }
    return render(request, "secretsanta/applications.html", context)


@login_required
@permission_required("secretsanta.basic_access")
def apply(request, year) -> HttpResponse:
    year_obj = Year.objects.get(year=year)
    if year_obj.open is not True:
        messages.error(request, _('This year is not open for applications'))
        return redirect("secretsanta:index")
    try:
        Application.objects.create(year=year_obj, user=request.user)
        messages.success(request, _("Joined Secret Santa for {year}").format(year=year))
        return redirect("secretsanta:index")
    except ValidationError:
        messages.error(request, _("You may only join once"))
    except Exception as e:
        logger.error(f"secretsanta.views.apply() {request.user} {e}")
        messages.error(request, _('An unexpected error ocurred '))
    return redirect("secretsanta:index")


@login_required
@permission_required("secretsanta.basic_access")
def mark_received(request, year) -> HttpResponse:
    try:
        year_obj = Year.objects.get(year=year)
        sp = SantaPair.objects.get(year=year_obj, santee=request.user)
        sp.delivered = True
        sp.save()
        messages.success(request, _("Gift marked as received for {year}").format(year=year))
        return redirect("secretsanta:index")
    except ValidationError:
        messages.error(request, _("You have already received your gift"))
    except Exception as e:
        logger.error(f"secretsanta.views.mark_received() {request.user} {e}")
        messages.error(request, _('An unexpected error ocurred'))
    return redirect("secretsanta:index")


@login_required
@permission_required("secretsanta.manager")
def admin_delete_application(request, id) -> HttpResponse:
    try:
        year_obj = Application.objects.get(id=id)
        year_obj.delete()
        messages.success(request, _("Deleted SS Application"))
        return redirect("secretsanta:index")
    except ValidationError:
        messages.error(request, _("You have already received your gift"))
    except Exception as e:
        logger.error(f"secretsanta.views.admin_delete_application() {request.user} {e}")
        messages.error(request, _('An unexpected error occurred'))
    return redirect("secretsanta:index")


@login_required
@permission_required("secretsanta.manager")
def queue_generate_pairs(request, year) -> HttpResponse:
    try:
        generate_pairs.apply_async(
            args=[year], priority=SECRETSANTA_GENERATE_PAIRS_PRIORITY)
        messages.success(request, _("Celery Task Queued to Generate Secret Santa for {year}").format(year=year))
        return redirect("secretsanta:index")
    except Exception as e:
        logger.error(f"secretsanta.views.queue_generate_pairs() {request.user} {e}")
        messages.error(request, _('An unexpected error ocurred'))
    return redirect("secretsanta:index")


@login_required
@permission_required("secretsanta.manager")
def queue_notify_santas(request, year) -> HttpResponse:
    try:
        notify_santas.apply_async(
            args=[year], priority=SECRETSANTA_NOTIFY_PRIORITY)
        messages.success(request, _("Celery Task queue_notify_santas for {year}").format(year=year))
        return redirect("secretsanta:index")
    except Exception as e:
        logger.error(f"secretsanta.views.queue_generate_pairs() {request.user} {e}")
        messages.error(request, _('An unexpected error ocurred'))
    return redirect("secretsanta:index")


@login_required
@permission_required("secretsanta.manager")
def queue_notify_outstanding_santees(request, year) -> HttpResponse:
    try:
        notify_outstanding_santees.apply_async(
            args=[year], priority=SECRETSANTA_NOTIFY_PRIORITY)
        messages.success(request, _("Celery Task queue_notify_outstanding_santees for {year}").format(year=year))
        return redirect("secretsanta:index")
    except Exception as e:
        logger.error(f"secretsanta.views.queue_generate_pairs() {request.user} {e}")
        messages.error(request, _('An unexpected error ocurred'))
    return redirect("secretsanta:index")



from django.contrib import admin

from .models import (
    ActiveSecretSantaFilter, Application, SantaPair, SecretSantaConfiguration,
    Webhook, Year,
)


@admin.register(Application)
class ApplicationAdmin(admin.ModelAdmin):
    list_display = ['year', 'user']


@admin.register(SantaPair)
class SantaPairAdmin(admin.ModelAdmin):
    list_display = ['year', 'santa', 'santee', 'delivered']


@admin.register(Webhook)
class WebhookAdmin(admin.ModelAdmin):
    list_display = ['name', 'url']


@admin.register(SecretSantaConfiguration)
class SecretSantaConfigurationAdmin(admin.ModelAdmin):
    list_display = ['id']


@admin.register(Year)
class YearAdmin(admin.ModelAdmin):
    list_display = ['year', 'open']


@admin.register(ActiveSecretSantaFilter)
class ActiveSecretSantaFilterAdmin(admin.ModelAdmin):
    list_display = ['year', 'reversed_logic']



# Generated by Django 4.2.6 on 2023-10-31 05:09

import django.core.validators
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='General',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('basic_access', 'Can access Secret Santa'), ('manager', 'Can Manage Secret Santa')),
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('url', models.URLField(verbose_name='URL')),
                ('enabled', models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name='Year',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField(validators=[django.core.validators.MinValueValidator(2000), django.core.validators.MaxValueValidator(2050)])),
                ('open', models.BooleanField(default=True, verbose_name='Open to Applications')),
            ],
        ),
        migrations.CreateModel(
            name='SecretSantaConfiguration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('delivery_updates', models.ManyToManyField(help_text='A Santee has marked they have received their gift', to='secretsanta.webhook', verbose_name='Delivery Updates')),
            ],
            options={
                'verbose_name': 'Secret Santa Configuration',
            },
        ),
        migrations.CreateModel(
            name='SantaPair',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('delivered', models.BooleanField(verbose_name='Delivered')),
                ('santa', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Santa')),
                ('santee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Santee')),
                ('year', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='secretsanta.year', verbose_name='Year')),
            ],
            options={
                'default_permissions': (),
                'unique_together': {('year', 'santee'), ('year', 'santa')},
            },
        ),
        migrations.CreateModel(
            name='Application',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
                ('year', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='secretsanta.year', verbose_name='Year')),
            ],
            options={
                'default_permissions': (),
                'unique_together': {('year', 'user')},
            },
        ),
    ]



from random import shuffle

from celery import shared_task

from allianceauth.services.hooks import get_extension_logger

from .app_settings import discordbot_active
from .models import Application, SantaPair, Year

logger = get_extension_logger(__name__)

if discordbot_active():
    from aadiscordbot.tasks import send_message


@shared_task
def generate_pairs(year: int):
    year_obj = Year.objects.get(year=year)
    applications = Application.objects.filter(year=year_obj)

    if applications.count() == 1:
        # This is impossible, exit early to prevent locking celery
        return

    applications_list = list(applications)
    pairs = []
    invalid_list = True
    while invalid_list:
        pairs = []
        shuffle(applications_list)
        for i, santa in enumerate(applications):
            santee = applications_list[i]
            if santa.user == santee.user:
                invalid_list = True
                break
            else:
                pairs.append((santa.user, santee.user))

        if len(pairs) == applications.count():
            invalid_list = False

    for pair in pairs:
        SantaPair.objects.create(
            santa=pair[0],
            santee=pair[1],
            year=year_obj
        )
    year_obj.open = False
    year_obj.save()


@shared_task
def notify_santas(year: Year) -> None:
    year_obj = Year.objects.get(year=year)
    for pair in SantaPair.objects.filter(year=year_obj):
        if discordbot_active():
            try:
                message = f"Hello {pair.santa.profile.main_character.character_name}, Your santee is `{pair.santee.profile.main_character.character_name}`, Happy Holidays"
                send_message(user=pair.santa, message=message)
            except Exception as e:
                logger.error(f"Fialed to ping Santa {e}", exc_info=True)
    pass


@shared_task
def notify_outstanding_santees(year: Year) -> None:
    year_obj = Year.objects.get(year=year)
    for pair in SantaPair.objects.filter(year=year_obj, delivered=False):
        if discordbot_active():
            message = f"Hello {pair.santee.profile.main_character.character_name} you have not marked your gift as received, please let leadership know if you haven't received anything, or update your Gift as received on Auth"
            send_message(user=pair.santee, message=message)
        pass



from django.utils.translation import gettext_lazy as _

from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

from . import models, urls


class SecretSantaMenuItem(MenuItemHook):
    """This class ensures only authorized users will see the menu entry"""

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            _("Secret Santa"),
            "fas fa-gifts fa-fw",
            "secretsanta:index",
            navactive=["secretsanta:"],
        )

    def render(self, request):
        if request.user.has_perm("secretsanta.basic_access"):
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu() -> SecretSantaMenuItem:
    return SecretSantaMenuItem()


@hooks.register("url_hook")
def register_urls() -> UrlHook:
    return UrlHook(urls, "secretsanta", r"^secretsanta/")


@hooks.register("secure_group_filters")
def filters():
    return [models.ActiveSecretSantaFilter]



from django.apps import AppConfig

from . import __version__


class SecretSantaConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = "secretsanta"
    verbose_name = f'AA Secret Santa v{__version__}'



import re

from django.apps import apps
from django.conf import settings


def get_site_url():  # regex sso url
    regex = r"^(.+)\/s.+"
    matches = re.finditer(regex, settings.ESI_SSO_CALLBACK_URL, re.MULTILINE)
    url = "http://"

    for m in matches:
        url = m.groups()[0]  # first match

    return url


def discordbot_active() -> bool:
    return apps.is_installed("aadiscordbot")


SECRETSANTA_GENERATE_PAIRS_PRIORITY = getattr(settings, "SECRETSANTA_GENERATE_PAIRS_PRIORITY", 1)
SECRETSANTA_NOTIFY_PRIORITY = getattr(settings, "SECRETSANTA_NOTIFY_PRIORITY", 5)



from django.urls import path

from . import views

app_name = "secretsanta"

urlpatterns = [
    path("", views.index, name="index"),
    path("apply/<int:year>/", views.apply, name="apply"),
    path("mark_received/<int:year>/", views.mark_received, name="mark_received"),
    path("pairs/<int:year>/", views.pairs, name="pairs"),
    path("applications/<int:year>/", views.applications, name="applications"),
    path("generate_pairs/<int:year>/", views.queue_generate_pairs, name="generate_pairs"),
    path("notify_santas/<int:year>/", views.queue_notify_santas, name="notify_santas"),
    path("notify_outstanding_santees/<int:year>/", views.queue_notify_outstanding_santees, name="notify_outstanding_santees"),
    path("admin_delete_application/<int:id>/", views.admin_delete_application, name="admin_delete_application"),
]



# Generated by Django 4.2.6 on 2023-10-31 09:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('secretsanta', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='santapair',
            name='delivered',
            field=models.BooleanField(default=False, verbose_name='Delivered'),
        ),
    ]



# Generated by Django 4.0 on 2023-10-31 14:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('secretsanta', '0002_alter_santapair_delivered'),
    ]

    operations = [
        migrations.AlterField(
            model_name='application',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='auth.user', verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='secretsantaconfiguration',
            name='delivery_updates',
            field=models.ManyToManyField(help_text='A Santee has marked they have received their gift', to='secretsanta.Webhook', verbose_name='Delivery Updates'),
        ),
    ]



from django.contrib.auth.models import User
from django.test import TestCase

from secretsanta.models import (
    ActiveSecretSantaFilter, Application, SantaPair, SecretSantaConfiguration,
    Webhook, Year,
)


class TestModels(TestCase):

    def setUp(self):
        self.user1 = User.objects.create_user(username='user1', password='password')
        self.user2 = User.objects.create_user(username='user2', password='password')

    def test_webhook_model(self):
        webhook = Webhook.objects.create(name='Test Webhook', url='http://testwebhook.com')
        self.assertEqual(str(webhook), 'Test Webhook')

    def test_secretsantaconfiguration_model(self):
        config = SecretSantaConfiguration.objects.create()
        self.assertEqual(str(config), 'Secret Santa Configuration')

    def test_year_model(self):
        year = Year.objects.create(year=2023)
        self.assertEqual(year.members, 0)

    def test_application_model(self):
        year = Year.objects.create(year=2023)
        application = Application.objects.create(year=year, user=self.user1)
        self.assertEqual(application.year, year)
        self.assertEqual(application.user, self.user1)

    def test_filter_model_process_normal(self):
        year = Year.objects.create(year=2023)
        application = Application.objects.create(year=year, user=self.user1)
        filter = ActiveSecretSantaFilter.objects.create(
            name="test 1",
            description="test1",
            year=year
        )
        self.assertEqual(application.year, year)
        self.assertEqual(application.user, self.user1)
        self.assertTrue(filter.process_filter(self.user1))
        self.assertFalse(filter.process_filter(self.user2))

    def test_filter_model_process_reverse(self):
        year = Year.objects.create(year=2023)
        application = Application.objects.create(year=year, user=self.user1)
        filter = ActiveSecretSantaFilter.objects.create(
            name="test 1",
            description="test1",
            year=year,
            reversed_logic=True
        )
        self.assertEqual(application.year, year)
        self.assertEqual(application.user, self.user1)
        self.assertFalse(filter.process_filter(self.user1))
        self.assertTrue(filter.process_filter(self.user2))

    def test_filter_model_audit_normal(self):
        year = Year.objects.create(year=2023)
        application = Application.objects.create(year=year, user=self.user1)
        filter = ActiveSecretSantaFilter.objects.create(
            name="test 1",
            description="test1",
            year=year
        )
        self.assertEqual(application.year, year)
        self.assertEqual(application.user, self.user1)
        a = filter.audit_filter([self.user1, self.user2])
        self.assertTrue(a[self.user1.id]["check"])
        self.assertFalse(a[self.user2.id]["check"])
        self.assertFalse(a[3]["check"])

    def test_filter_model_audit_reverse(self):
        year = Year.objects.create(year=2023)
        application = Application.objects.create(year=year, user=self.user1)
        filter = ActiveSecretSantaFilter.objects.create(
            name="test 1",
            description="test1",
            year=year,
            reversed_logic=True
        )
        self.assertEqual(application.year, year)
        self.assertEqual(application.user, self.user1)
        a = filter.audit_filter([self.user1, self.user2])
        self.assertFalse(a[self.user1.id]["check"])
        self.assertTrue(a[self.user2.id]["check"])
        self.assertTrue(a[3]["check"])

    def test_santapair_model(self):
        year = Year.objects.create(year=2023)
        pair = SantaPair.objects.create(year=year, santa=self.user1, santee=self.user2)
        self.assertEqual(pair.year, year)
        self.assertEqual(pair.santa, self.user1)
        self.assertEqual(pair.santee, self.user2)



# Generated by Django 4.2.6 on 2023-11-01 02:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('secretsanta', '0003_alter_application_user_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='secretsantaconfiguration',
            options={'default_permissions': (), 'verbose_name': 'Secret Santa Configuration'},
        ),
    ]



# Generated by Django 4.2.8 on 2023-12-11 10:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('secretsanta', '0004_alter_secretsantaconfiguration_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='ActiveSecretSantaFilter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('reversed_logic', models.BooleanField(default=False, help_text='If set all members WITHOUT a secret santa application will pass the test.')),
                ('year', models.ForeignKey(help_text='What year to filter on.', on_delete=django.db.models.deletion.CASCADE, to='secretsanta.year')),
            ],
            options={
                'verbose_name': 'Smart Filter: Active Secret Santa',
                'verbose_name_plural': 'Smart Filter: Active Secret Santa',
                'default_permissions': (),
            },
        ),
    ]



from django.contrib.auth.models import User
from django.test import TestCase

from secretsanta.models import Application, SantaPair, Year
from secretsanta.tasks import (
    generate_pairs, notify_outstanding_santees, notify_santas,
)


class TestTasks(TestCase):

    def setUp(self):
        self.user1 = User.objects.create_user(username='user1', password='password')
        self.user2 = User.objects.create_user(username='user2', password='password')
        self.user3 = User.objects.create_user(username='user3', password='password')
        self.user4 = User.objects.create_user(username='user4', password='password')
        self.user5 = User.objects.create_user(username='user5', password='password')
        self.user6 = User.objects.create_user(username='user6', password='password')
        self.user7 = User.objects.create_user(username='user7', password='password')
        self.user8 = User.objects.create_user(username='user8', password='password')
        self.user9 = User.objects.create_user(username='user9', password='password')
        self.user10 = User.objects.create_user(username='user10', password='password')

        self.year1 = Year.objects.create(year=2023, open=True)

    def test_generate_pairs(self):
        year = self.year1
        Application.objects.create(year=year, user=self.user1)
        Application.objects.create(year=year, user=self.user2)
        Application.objects.create(year=year, user=self.user3)
        Application.objects.create(year=year, user=self.user4)
        Application.objects.create(year=year, user=self.user5)
        Application.objects.create(year=year, user=self.user6)
        Application.objects.create(year=year, user=self.user7)
        Application.objects.create(year=year, user=self.user8)
        Application.objects.create(year=year, user=self.user9)
        Application.objects.create(year=year, user=self.user10)
        generate_pairs(year.year)
        self.assertEqual(SantaPair.objects.count(), 10)

    def test_notify_santas(self):
        year = self.year1
        SantaPair.objects.create(year=year, santa=self.user1, santee=self.user2)
        notify_santas(year.year)
        # Here you can check if the message was sent correctly

    def test_notify_outstanding_santees(self):
        year = self.year1
        SantaPair.objects.create(year=year, santa=self.user1, santee=self.user2, delivered=False)
        notify_outstanding_santees(year.year)
        # Here you can check if the message was sent correctly
